/* Sistema Caotico: equacao de diferenca logistica

                 2  
X    =  KX  -  KX
 i+1      i      i

Xi = tamanho da populacao num instante i
K  = parametro de crescimento da populacao (constante)

Para compilar:
gcc logist.cc -lXaw -lXmu -lXext -lXt -lX11 -ldl -lm -L /usr/X11/lib */
     
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>
#include <sys/timeb.h>

#define XK_MISCELLANY 1
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysymdef.h>

#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Box.h>

#ifdef DO_SHM
#include <X11/extensions/XShm.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#endif /* DO_SHM */

#define RESX 400
#define RESY 400

#define DEPTH 8
Display* pDpy = NULL;
XtAppContext stApContext;
Widget stTopLevel = 0, stShell = 0, stBox;

int nScreenNum;
int nDpyWidth, nDpyHeight;
Window stWindow;
XImage* pXImg = NULL;
GC stGC;

#ifdef DO_SHM
static XShmSegmentInfo stShmi;
XImage stShmImg;
#endif

unsigned char *pVid = NULL;

Colormap stColorMap = 0;

static XrmOptionDescRec table[] = {
    {"-flag",   "Flag",                XrmoptionNoArg,  (caddr_t) "on"},
    {"-param",  "parameter",           XrmoptionSepArg, NULL},
};

typedef struct {
    int flag;
    int parameter;
    char *geometry;
    int borderwidth;
} stAppResourceRec, *pAppRes;

static stAppResourceRec stAppResources;
static char szWinDimension [ 100 ];

static XtResource resources[] = {
 {"param", "Parameter", XtRInt, sizeof(int),
   XtOffset(pAppRes, parameter), XtRImmediate, (caddr_t) 80},
 {"flag", "Flag", XtRBoolean, sizeof(Boolean),
   XtOffset(pAppRes, flag), XtRImmediate, (caddr_t) TRUE },
 {XtNgeometry,  "Geometry", XtRString, sizeof(caddr_t),
   XtOffset(pAppRes, geometry), XtRString, (caddr_t) szWinDimension},
 {XtNborderWidth, "borderWidth", XtRInt, sizeof(int),
   XtOffset(pAppRes, borderwidth), XtRImmediate, (caddr_t) 1},
};



// Variaveis do sistema caotico:
float
    fXi = 0.1;


/* Atualiza area de video
**********************************************************/
void UpDate ()
{
  # ifdef DO_SHM
    XShmPutImage (pDpy, stWindow, stGC, &stShmImg, 0, 0, 0, 0, RESX, RESY, False);
  # endif
  XPutImage (pDpy, stWindow, stGC, pXImg, 0, 0, 0, 0, RESX, RESY);
}



/* Marca um pixel na area de video
**********************************************************/
void SetPixel ( int x, int y, int color )
{
  if ( x < 0 || x >= RESX ||
       y < 0 || y >= RESY )
    return;

  * ( pVid + RESX * y +x ) = color;
}



/* Limpa area de video
**********************************************************/
void Clear (int color)
{
  if ( pVid )
    memset ( pVid, color, RESX * RESY );
}



/* Grafico da variacao da populacao
**********************************************************/
void Grafico ()
{
  // limpa janela
  Clear ( 0 );

  // faz grafico
  for ( int ncT = 1; ncT < RESX; ncT ++ ) {

    float
      fK = (float) ncT / RESX * 4;
    fXi = fK * fXi - fK * ( fXi * fXi );
    SetPixel ( ncT, (int) ( fXi * RESY ), 1 );
  }

  UpDate ();
}



/* Termina janela
**********************************************************/
Term ()
{
  if ( stShell )
    XtPopdown ( stShell );
  if ( stTopLevel )
    XtDestroyWidget ( stTopLevel );
}



/* Trata eventos do X Windows
**********************************************************/
void EventsHandler (
  Widget w,
  XtPointer closure,
  XEvent* event,
  Boolean* cont_to_dispatch)
{
  KeySym key;
  int key2;
  int state;
  
  //(void)w; (void)closure; (void)cont_to_dispatch;

  switch ( event->type )
  {
    case Expose:
      if ( ! pXImg )
      {
	pXImg = XGetImage (pDpy, stWindow, 0, 0, RESX, RESY, AllPlanes, ZPixmap);
        #ifdef DO_SHM
	{
	  stShmImg = *pXImg;
	  stShmi.shmid = shmget (IPC_PRIVATE, pXImg->bytes_per_line*pXImg->height,
			       IPC_CREAT|0777);
	  stShmi.shmaddr = (char*)shmat (stShmi.shmid, 0, 0);
	  stShmi.readOnly = FALSE;
	  XShmAttach (pDpy, &stShmi);
	  stShmImg.data = stShmi.shmaddr;
	  pVid = (unsigned char*)stShmi.shmaddr;
	  stShmImg.obdata = (char*)&stShmi;
	}
        #endif /* DO_SHM */
	{
	  pVid = (unsigned char*)(pXImg->data);
        }
      }
      // faz o grafico
      Grafico ();
      break;

    case KeyPress:
      key = XLookupKeysym ((XKeyEvent*)event, 0);
      state = ((XKeyEvent*)event)->state;
      key2 = key;
      switch (key)
      {
	case XK_Escape:
	  Term ();
	  exit ( 0 );
	  break;
	default:
          break;
      }
      break;

    default:
      break;
  }
}



/* Inicia o aplicativo
**********************************************************/
IniApl ( int argc, char* argv [] )
{
  // cria janela, palette, define eventos, ...

  XGCValues stValues;
  Screen* pScreenPtr;
  Widget stBox;

  sprintf (szWinDimension, "%dx%d", RESX, RESY);

  stTopLevel = XtVaAppInitialize (&stApContext, "Diferenca Logistica", table, XtNumber (table),
	&argc, argv, NULL, NULL, 0);

  XtGetApplicationResources (stTopLevel, &stAppResources, resources,
	XtNumber(resources), NULL, (Cardinal) 0);
  /*XParseGeometry (stAppResources.geometry, &geom_x, &geom_y, &geom_width,
	&geom_height);*/

  stShell = XtVaCreatePopupShell ("Diferenca Logistica",
				topLevelShellWidgetClass,
				stTopLevel,
				XtNgeometry, stAppResources.geometry,
				XtNborderWidth, stAppResources.borderwidth,
				0);
  /*stBox = XtCreateManagedWidget ("box", boxWidgetClass, stShell, NULL, (Cardinal) 0);*/
  stBox = XtCreateWidget ("box", boxWidgetClass, stShell, NULL, (Cardinal) 0);

  XtPopup (stShell, XtGrabNone);
 
  pDpy = XtDisplay (stShell);
  nScreenNum = DefaultScreen (pDpy);
  pScreenPtr = DefaultScreenOfDisplay (pDpy);
  nDpyWidth = DisplayWidth (pDpy, nScreenNum);
  nDpyHeight = DisplayHeight (pDpy, nScreenNum);
  stWindow = XtWindow (stShell);

  stGC = XCreateGC (pDpy, stWindow, 0, &stValues);
  XSetForeground (pDpy, stGC, BlackPixel (pDpy, nScreenNum));
  XSetLineAttributes (pDpy, stGC, 0, LineSolid, CapButt, JoinMiter);
  XSetGraphicsExposures (pDpy, stGC, False);

  stColorMap = XCreateColormap (pDpy, RootWindow (pDpy, nScreenNum),
			  DefaultVisual (pDpy, nScreenNum), AllocAll);

  XtAddEventHandler (stShell, ExposureMask | KeyPressMask | KeyReleaseMask |
  		     KeymapStateMask | FocusChangeMask |
		     PointerMotionMask | ButtonPressMask | ButtonReleaseMask, True,
		     EventsHandler, NULL);

  XMapWindow (pDpy, XtWindow (stShell));
  XSetWindowColormap (pDpy, XtWindow (stShell), stColorMap);

  pXImg = NULL;

  // mensagem de inicio
  printf ( "Sistema Caotico: equacao da diferenca logistica.\n");
  #ifdef DO_SHM
  printf ( "Usando SHM\n" );
  #endif
  #ifndef DO_SHM
    printf ( "Sem SHM\n" );
  #endif

  // pede parametros
  printf ( "\nEntre Xi inicial (0 <= Xi <= 1, ex.: 0.000076): " );
  scanf ( "%f", &fXi );
  printf ( "\nXi = %f\n", fXi );
  printf ( "Durante a execucao tecle ESC para sai.\n" );
  
  // programa palette
  XColor
    color;
  for ( int ncI = 0 ; ncI < 256; ncI ++ ) {
    color.pixel = ncI;
    color.red = 64 * 256;
    color.green = ( ( ncI * 20 ) % 256 ) * 256;
    color.blue = 255 * 256;
    color.flags = DoRed | DoGreen | DoBlue;
    XStoreColor ( pDpy, stColorMap, &color );
  }
  // fundo
  color.pixel = 0;
  color.red = 0 * 256;
  color.green = 0 * 256;
  color.blue = 0 * 256;
  color.flags = DoRed | DoGreen | DoBlue;
  XStoreColor ( pDpy, stColorMap, &color );
  // grafico
  color.pixel = 1;
  color.red = 255 * 256;
  color.green = 255 * 256;
  color.blue = 255 * 256;
  color.flags = DoRed | DoGreen | DoBlue;
  XStoreColor ( pDpy, stColorMap, &color );
}



main ( int argc, char* argv [] )
{
  IniApl ( argc, argv );

  XtAppMainLoop ( stApContext );

  Term ();
}
